{
    "$graph": [
        {
            "class": "Workflow",
            "doc": "RNA-seq 04 quantification",
            "requirements": [
                {
                    "class": "ScatterFeatureRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "SubworkflowFeatureRequirement"
                }
            ],
            "inputs": [
                {
                    "doc": "GTF annotation file",
                    "type": "File",
                    "id": "#main/annotation_file"
                },
                {
                    "doc": "JSON filter file for forward strand used in bamtools (see bamtools-filter command)",
                    "type": "File",
                    "id": "#main/bamtools_forward_filter_file"
                },
                {
                    "doc": "JSON filter file for reverse strand used in bamtools (see bamtools-filter command)",
                    "type": "File",
                    "id": "#main/bamtools_reverse_filter_file"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "id": "#main/input_bam_files"
                },
                {
                    "type": "File",
                    "id": "#main/input_genome_sizes"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "id": "#main/input_transcripts_bam_files"
                },
                {
                    "default": 1,
                    "type": "int",
                    "id": "#main/nthreads"
                },
                {
                    "doc": "RSEM genome reference files - generated with the rsem-prepare-reference command",
                    "type": "Directory",
                    "id": "#main/rsem_reference_files"
                }
            ],
            "steps": [
                {
                    "run": "#deeptools-bamcoverage.cwl",
                    "scatter": "#main/bamcoverage-minus/bam",
                    "in": [
                        {
                            "source": "#main/split_bams/bam_minus_files",
                            "id": "#main/bamcoverage-minus/bam"
                        },
                        {
                            "valueFrom": "${return 1}",
                            "id": "#main/bamcoverage-minus/binSize"
                        },
                        {
                            "valueFrom": "RPKM",
                            "id": "#main/bamcoverage-minus/normalizeUsing"
                        },
                        {
                            "source": "#main/nthreads",
                            "id": "#main/bamcoverage-minus/numberOfProcessors"
                        },
                        {
                            "valueFrom": ".norm-minus-pre-negated-bw",
                            "id": "#main/bamcoverage-minus/output_suffix"
                        }
                    ],
                    "out": [
                        "#main/bamcoverage-minus/output_bam_coverage"
                    ],
                    "id": "#main/bamcoverage-minus"
                },
                {
                    "run": "#deeptools-bamcoverage.cwl",
                    "scatter": "#main/bamcoverage-plus/bam",
                    "in": [
                        {
                            "source": "#main/split_bams/bam_plus_files",
                            "id": "#main/bamcoverage-plus/bam"
                        },
                        {
                            "valueFrom": "${return 1}",
                            "id": "#main/bamcoverage-plus/binSize"
                        },
                        {
                            "valueFrom": "RPKM",
                            "id": "#main/bamcoverage-plus/normalizeUsing"
                        },
                        {
                            "source": "#main/nthreads",
                            "id": "#main/bamcoverage-plus/numberOfProcessors"
                        },
                        {
                            "valueFrom": ".norm.bw",
                            "id": "#main/bamcoverage-plus/output_suffix"
                        }
                    ],
                    "out": [
                        "#main/bamcoverage-plus/output_bam_coverage"
                    ],
                    "id": "#main/bamcoverage-plus"
                },
                {
                    "run": "#basename.cwl",
                    "in": [
                        {
                            "source": "#main/input_bam_files",
                            "valueFrom": "$(self.basename)",
                            "id": "#main/basename/file_path"
                        },
                        {
                            "valueFrom": "\\.Aligned\\.out\\.sorted",
                            "id": "#main/basename/sep"
                        }
                    ],
                    "scatter": "#main/basename/file_path",
                    "out": [
                        "#main/basename/basename"
                    ],
                    "id": "#main/basename"
                },
                {
                    "run": "#bedGraphToBigWig.cwl",
                    "scatter": "#main/bdg2bw-norm-minus/bed_graph",
                    "in": [
                        {
                            "source": "#main/negate_minus_bdg_norm/negated_minus_bdg",
                            "id": "#main/bdg2bw-norm-minus/bed_graph"
                        },
                        {
                            "source": "#main/input_genome_sizes",
                            "id": "#main/bdg2bw-norm-minus/genome_sizes"
                        },
                        {
                            "valueFrom": ".Aligned.minus.norm.bw",
                            "id": "#main/bdg2bw-norm-minus/output_suffix"
                        }
                    ],
                    "out": [
                        "#main/bdg2bw-norm-minus/output_bigwig"
                    ],
                    "id": "#main/bdg2bw-norm-minus"
                },
                {
                    "run": "#bedGraphToBigWig.cwl",
                    "scatter": "#main/bdg2bw-raw-minus/bed_graph",
                    "in": [
                        {
                            "source": "#main/negate_minus_bdg/negated_minus_bdg",
                            "id": "#main/bdg2bw-raw-minus/bed_graph"
                        },
                        {
                            "source": "#main/input_genome_sizes",
                            "id": "#main/bdg2bw-raw-minus/genome_sizes"
                        },
                        {
                            "valueFrom": ".bw",
                            "id": "#main/bdg2bw-raw-minus/output_suffix"
                        }
                    ],
                    "out": [
                        "#main/bdg2bw-raw-minus/output_bigwig"
                    ],
                    "id": "#main/bdg2bw-raw-minus"
                },
                {
                    "run": "#bedGraphToBigWig.cwl",
                    "scatter": "#main/bdg2bw-raw-plus/bed_graph",
                    "in": [
                        {
                            "source": "#main/bedsort_genomecov_plus/bed_file_sorted",
                            "id": "#main/bdg2bw-raw-plus/bed_graph"
                        },
                        {
                            "source": "#main/input_genome_sizes",
                            "id": "#main/bdg2bw-raw-plus/genome_sizes"
                        },
                        {
                            "valueFrom": ".raw.bw",
                            "id": "#main/bdg2bw-raw-plus/output_suffix"
                        }
                    ],
                    "out": [
                        "#main/bdg2bw-raw-plus/output_bigwig"
                    ],
                    "id": "#main/bdg2bw-raw-plus"
                },
                {
                    "run": "#bedSort.cwl",
                    "scatter": "#main/bedsort_genomecov_minus/bed_file",
                    "in": [
                        {
                            "source": "#main/bedtools_genomecov_minus/output_bedfile",
                            "id": "#main/bedsort_genomecov_minus/bed_file"
                        }
                    ],
                    "out": [
                        "#main/bedsort_genomecov_minus/bed_file_sorted"
                    ],
                    "id": "#main/bedsort_genomecov_minus"
                },
                {
                    "run": "#bedSort.cwl",
                    "scatter": "#main/bedsort_genomecov_plus/bed_file",
                    "in": [
                        {
                            "source": "#main/bedtools_genomecov_plus/output_bedfile",
                            "id": "#main/bedsort_genomecov_plus/bed_file"
                        }
                    ],
                    "out": [
                        "#main/bedsort_genomecov_plus/bed_file_sorted"
                    ],
                    "id": "#main/bedsort_genomecov_plus"
                },
                {
                    "run": "#bedtools-genomecov.cwl",
                    "scatter": "#main/bedtools_genomecov_minus/ibam",
                    "in": [
                        {
                            "valueFrom": "${return true}",
                            "id": "#main/bedtools_genomecov_minus/bg"
                        },
                        {
                            "source": "#main/input_genome_sizes",
                            "id": "#main/bedtools_genomecov_minus/g"
                        },
                        {
                            "source": "#main/split_bams/bam_minus_files",
                            "id": "#main/bedtools_genomecov_minus/ibam"
                        }
                    ],
                    "out": [
                        "#main/bedtools_genomecov_minus/output_bedfile"
                    ],
                    "id": "#main/bedtools_genomecov_minus"
                },
                {
                    "run": "#bedtools-genomecov.cwl",
                    "scatter": "#main/bedtools_genomecov_plus/ibam",
                    "in": [
                        {
                            "valueFrom": "${return true}",
                            "id": "#main/bedtools_genomecov_plus/bg"
                        },
                        {
                            "source": "#main/input_genome_sizes",
                            "id": "#main/bedtools_genomecov_plus/g"
                        },
                        {
                            "source": "#main/split_bams/bam_plus_files",
                            "id": "#main/bedtools_genomecov_plus/ibam"
                        }
                    ],
                    "out": [
                        "#main/bedtools_genomecov_plus/output_bedfile"
                    ],
                    "id": "#main/bedtools_genomecov_plus"
                },
                {
                    "run": "#bigWigToBedGraph.cwl",
                    "scatter": "#main/bw2bdg-minus/bigwig_file",
                    "in": [
                        {
                            "source": "#main/bamcoverage-minus/output_bam_coverage",
                            "id": "#main/bw2bdg-minus/bigwig_file"
                        }
                    ],
                    "out": [
                        "#main/bw2bdg-minus/output_bedgraph"
                    ],
                    "id": "#main/bw2bdg-minus"
                },
                {
                    "run": "#subread-featurecounts.cwl",
                    "in": [
                        {
                            "valueFrom": "${return true}",
                            "id": "#main/featurecounts/B"
                        },
                        {
                            "source": "#main/nthreads",
                            "id": "#main/featurecounts/T"
                        },
                        {
                            "source": "#main/annotation_file",
                            "id": "#main/featurecounts/annotation_file"
                        },
                        {
                            "valueFrom": "gene_id",
                            "id": "#main/featurecounts/g"
                        },
                        {
                            "source": "#main/input_bam_files",
                            "valueFrom": "${if (Array.isArray(self)) return self; return [self]; }",
                            "id": "#main/featurecounts/input_files"
                        },
                        {
                            "source": "#main/basename/basename",
                            "valueFrom": "$(self + \".featurecounts.counts.txt\")",
                            "id": "#main/featurecounts/output_filename"
                        },
                        {
                            "valueFrom": "${return true}",
                            "id": "#main/featurecounts/p"
                        },
                        {
                            "valueFrom": "${return 0}",
                            "id": "#main/featurecounts/s"
                        },
                        {
                            "valueFrom": "exon",
                            "id": "#main/featurecounts/t"
                        }
                    ],
                    "scatterMethod": "dotproduct",
                    "scatter": [
                        "#main/featurecounts/input_files",
                        "#main/featurecounts/output_filename"
                    ],
                    "out": [
                        "#main/featurecounts/output_files"
                    ],
                    "id": "#main/featurecounts"
                },
                {
                    "run": "#negate-minus-strand-bedgraph.cwl",
                    "scatterMethod": "dotproduct",
                    "scatter": [
                        "#main/negate_minus_bdg/bedgraph_file",
                        "#main/negate_minus_bdg/output_filename"
                    ],
                    "in": [
                        {
                            "source": "#main/bedsort_genomecov_minus/bed_file_sorted",
                            "id": "#main/negate_minus_bdg/bedgraph_file"
                        },
                        {
                            "source": "#main/basename/basename",
                            "valueFrom": "$(self + \".Aligned.minus.raw.bdg\")",
                            "id": "#main/negate_minus_bdg/output_filename"
                        }
                    ],
                    "out": [
                        "#main/negate_minus_bdg/negated_minus_bdg"
                    ],
                    "id": "#main/negate_minus_bdg"
                },
                {
                    "run": "#negate-minus-strand-bedgraph.cwl",
                    "scatter": [
                        "#main/negate_minus_bdg_norm/bedgraph_file",
                        "#main/negate_minus_bdg_norm/output_filename"
                    ],
                    "scatterMethod": "dotproduct",
                    "in": [
                        {
                            "source": "#main/bw2bdg-minus/output_bedgraph",
                            "id": "#main/negate_minus_bdg_norm/bedgraph_file"
                        },
                        {
                            "source": "#main/basename/basename",
                            "valueFrom": "$(self + \".norm-minus-bdg\")",
                            "id": "#main/negate_minus_bdg_norm/output_filename"
                        }
                    ],
                    "out": [
                        "#main/negate_minus_bdg_norm/negated_minus_bdg"
                    ],
                    "id": "#main/negate_minus_bdg_norm"
                },
                {
                    "run": "#rsem-calculate-expression.cwl",
                    "in": [
                        {
                            "source": "#main/input_transcripts_bam_files",
                            "id": "#main/rsem-calc-expr/bam"
                        },
                        {
                            "valueFrom": "${return true}",
                            "id": "#main/rsem-calc-expr/no-bam-output"
                        },
                        {
                            "source": "#main/nthreads",
                            "id": "#main/rsem-calc-expr/num-threads"
                        },
                        {
                            "valueFrom": "${return true}",
                            "id": "#main/rsem-calc-expr/paired-end"
                        },
                        {
                            "valueFrom": "${return true}",
                            "id": "#main/rsem-calc-expr/quiet"
                        },
                        {
                            "source": "#main/rsem_reference_files",
                            "id": "#main/rsem-calc-expr/reference_files"
                        },
                        {
                            "source": "#main/rsem_reference_files",
                            "valueFrom": "${\n  var trans_file_str = self.listing.map(function(e){return e.location}).filter(function(e){return e.match(/\\.transcripts\\.fa$/)})[0];\n  return trans_file_str.match(/.*[\\\\\\\\\\\\/](.*)\\.transcripts\\.fa$/)[1];\n}\n",
                            "id": "#main/rsem-calc-expr/reference_name"
                        },
                        {
                            "source": "#main/basename/basename",
                            "valueFrom": "$(self + \".rsem\")",
                            "id": "#main/rsem-calc-expr/sample_name"
                        },
                        {
                            "valueFrom": "${return 1234}",
                            "id": "#main/rsem-calc-expr/seed"
                        }
                    ],
                    "scatterMethod": "dotproduct",
                    "scatter": [
                        "#main/rsem-calc-expr/bam",
                        "#main/rsem-calc-expr/sample_name"
                    ],
                    "out": [
                        "#main/rsem-calc-expr/isoforms",
                        "#main/rsem-calc-expr/genes",
                        "#main/rsem-calc-expr/rsem_stat"
                    ],
                    "id": "#main/rsem-calc-expr"
                },
                {
                    "in": [
                        {
                            "source": "#main/bamtools_forward_filter_file",
                            "id": "#main/split_bams/bamtools_forward_filter_file"
                        },
                        {
                            "source": "#main/bamtools_reverse_filter_file",
                            "id": "#main/split_bams/bamtools_reverse_filter_file"
                        },
                        {
                            "source": "#main/input_bam_files",
                            "id": "#main/split_bams/input_bam_files"
                        },
                        {
                            "source": "#main/basename/basename",
                            "id": "#main/split_bams/input_basenames"
                        }
                    ],
                    "run": "#split-bams-by-strand-and-index.cwl",
                    "out": [
                        "#main/split_bams/bam_plus_files",
                        "#main/split_bams/bam_minus_files"
                    ],
                    "id": "#main/split_bams"
                }
            ],
            "id": "#main",
            "outputs": [
                {
                    "doc": "BAM files containing only reads in the reverse (minus) strand.",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/split_bams/bam_minus_files",
                    "id": "#main/bam_minus_files"
                },
                {
                    "doc": "BAM files containing only reads in the forward (plus) strand.",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/split_bams/bam_plus_files",
                    "id": "#main/bam_plus_files"
                },
                {
                    "doc": "Normalized by RPKM bigWig files from BAM files containing only reads in the forward (plus) strand.",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/bdg2bw-norm-minus/output_bigwig",
                    "id": "#main/bw_norm_minus_files"
                },
                {
                    "doc": "Normalized by RPKM bigWig files from BAM files containing only reads in the forward (plus) strand.",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/bamcoverage-plus/output_bam_coverage",
                    "id": "#main/bw_norm_plus_files"
                },
                {
                    "doc": "Raw bigWig files from BAM files containing only reads in the reverse (minus) strand.",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/bdg2bw-raw-minus/output_bigwig",
                    "id": "#main/bw_raw_minus_files"
                },
                {
                    "doc": "Raw bigWig files from BAM files containing only reads in the forward (plus) strand.",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/bdg2bw-raw-plus/output_bigwig",
                    "id": "#main/bw_raw_plus_files"
                },
                {
                    "doc": "Normalized fragment extended reads bigWig (signal) files",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/featurecounts/output_files",
                    "id": "#main/featurecounts_counts"
                },
                {
                    "doc": "RSEM genes files",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/rsem-calc-expr/genes",
                    "id": "#main/rsem_genes_files"
                },
                {
                    "doc": "RSEM isoforms files",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/rsem-calc-expr/isoforms",
                    "id": "#main/rsem_isoforms_files"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "doc": "Tool:    bedtools genomecov (aka genomeCoverageBed)\nVersion: v2.25.0\nSummary: Compute the coverage of a feature file among a genome.\n\nUsage: bedtools genomecov [OPTIONS] -i <bed/gff/vcf> -g <genome>\n\nOptions: \n\t-ibam\t\tThe input file is in BAM format.\n\t\t\tNote: BAM _must_ be sorted by position\n\n\t-d\t\tReport the depth at each genome position (with one-based coordinates).\n\t\t\tDefault behavior is to report a histogram.\n\n\t-dz\t\tReport the depth at each genome position (with zero-based coordinates).\n\t\t\tReports only non-zero positions.\n\t\t\tDefault behavior is to report a histogram.\n\n\t-bg\t\tReport depth in BedGraph format. For details, see:\n\t\t\tgenome.ucsc.edu/goldenPath/help/bedgraph.html\n\n\t-bga\t\tReport depth in BedGraph format, as above (-bg).\n\t\t\tHowever with this option, regions with zero \n\t\t\tcoverage are also reported. This allows one to\n\t\t\tquickly extract all regions of a genome with 0 \n\t\t\tcoverage by applying: \"grep -w 0$\" to the output.\n\n\t-split\t\tTreat \"split\" BAM or BED12 entries as distinct BED intervals.\n\t\t\twhen computing coverage.\n\t\t\tFor BAM files, this uses the CIGAR \"N\" and \"D\" operations \n\t\t\tto infer the blocks for computing coverage.\n\t\t\tFor BED12 files, this uses the BlockCount, BlockStarts, and BlockEnds\n\t\t\tfields (i.e., columns 10,11,12).\n\n\t-strand\t\tCalculate coverage of intervals from a specific strand.\n\t\t\tWith BED files, requires at least 6 columns (strand is column 6). \n\t\t\t- (STRING): can be + or -\n\n\t-5\t\tCalculate coverage of 5\" positions (instead of entire interval).\n\n\t-3\t\tCalculate coverage of 3\" positions (instead of entire interval).\n\n\t-max\t\tCombine all positions with a depth >= max into\n\t\t\ta single bin in the histogram. Irrelevant\n\t\t\tfor -d and -bedGraph\n\t\t\t- (INTEGER)\n\n\t-scale\t\tScale the coverage by a constant factor.\n\t\t\tEach coverage value is multiplied by this factor before being reported.\n\t\t\tUseful for normalizing coverage by, e.g., reads per million (RPM).\n\t\t\t- Default is 1.0; i.e., unscaled.\n\t\t\t- (FLOAT)\n\n\t-trackline\tAdds a UCSC/Genome-Browser track line definition in the first line of the output.\n\t\t\t- See here for more details about track line definition:\n\t\t\t      http://genome.ucsc.edu/goldenPath/help/bedgraph.html\n\t\t\t- NOTE: When adding a trackline definition, the output BedGraph can be easily\n\t\t\t      uploaded to the Genome Browser as a custom track,\n\t\t\t      BUT CAN NOT be converted into a BigWig file (w/o removing the first line).\n\n\t-trackopts\tWrites additional track line definition parameters in the first line.\n\t\t\t- Example:\n\t\t\t   -trackopts 'name=\"My Track\" visibility=2 color=255,30,30'\n\t\t\t   Note the use of single-quotes if you have spaces in your parameters.\n\t\t\t- (TEXT)\n\nNotes: \n\t(1) The genome file should tab delimited and structured as follows:\n\t <chromName><TAB><chromSize>\n\n\tFor example, Human (hg19):\n\tchr1\t249250621\n\tchr2\t243199373\n\t...\n\tchr18_gl000207_random\t4262\n\n\t(2) The input BED (-i) file must be grouped by chromosome.\n\t A simple \"sort -k 1,1 <BED> > <BED>.sorted\" will suffice.\n\n\t(3) The input BAM (-ibam) file must be sorted by position.\n\t A \"samtools sort <BAM>\" should suffice.\n\nTips: \n\tOne can use the UCSC Genome Browser's MySQL database to extract\n\tchromosome sizes. For example, H. sapiens:\n\n\tmysql --user=genome --host=genome-mysql.cse.ucsc.edu -A -e \\\n\t\"select chrom, size from hg19.chromInfo\" > hg19.genome",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ShellCommandRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "dukegcb/bedtools",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-3"
                    },
                    "doc": "\tCalculate coverage of 3\" positions (instead of entire interval).\n",
                    "id": "#bedtools-genomecov.cwl/3"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-5"
                    },
                    "doc": "\tCalculate coverage of 5\" positions (instead of entire interval).\n",
                    "id": "#bedtools-genomecov.cwl/5"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-bg"
                    },
                    "doc": "\tReport depth in BedGraph format. For details, see:\ngenome.ucsc.edu/goldenPath/help/bedgraph.html\n",
                    "id": "#bedtools-genomecov.cwl/bg"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-bga"
                    },
                    "doc": "\tReport depth in BedGraph format, as above (-bg).\nHowever with this option, regions with zero\ncoverage are also reported. This allows one to\nquickly extract all regions of a genome with 0\ncoverage by applying: \"grep -w 0$\" to the output.\n",
                    "id": "#bedtools-genomecov.cwl/bga"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-d"
                    },
                    "doc": "\tReport the depth at each genome position (with one-based coordinates).\nDefault behavior is to report a histogram.\n",
                    "id": "#bedtools-genomecov.cwl/d"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-dz"
                    },
                    "doc": "\tReport the depth at each genome position (with zero-based coordinates).\nReports only non-zero positions.\nDefault behavior is to report a histogram.\n",
                    "id": "#bedtools-genomecov.cwl/dz"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 3,
                        "prefix": "-g"
                    },
                    "doc": "<genome sizes>",
                    "id": "#bedtools-genomecov.cwl/g"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 2,
                        "prefix": "-ibam"
                    },
                    "doc": "\tThe input file is in BAM format.\nNote: BAM _must_ be sorted by position\n",
                    "id": "#bedtools-genomecov.cwl/ibam"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-max"
                    },
                    "doc": "\tCombine all positions with a depth >= max into\na single bin in the histogram. Irrelevant\nfor -d and -bedGraph\n- (INTEGER)\n",
                    "id": "#bedtools-genomecov.cwl/max"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-scale"
                    },
                    "doc": "\tScale the coverage by a constant factor.\nEach coverage value is multiplied by this factor before being reported.\nUseful for normalizing coverage by, e.g., reads per million (RPM).\n- Default is 1.0; i.e., unscaled.\n- (FLOAT)\n",
                    "id": "#bedtools-genomecov.cwl/scale"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-split"
                    },
                    "doc": "\tTreat \"split\" BAM or BED12 entries as distinct BED intervals.\nwhen computing coverage.\nFor BAM files, this uses the CIGAR \"N\" and \"D\" operations\nto infer the blocks for computing coverage.\nFor BED12 files, this uses the BlockCount, BlockStarts, and BlockEnds\nfields (i.e., columns 10,11,12).\n",
                    "id": "#bedtools-genomecov.cwl/split"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-strand"
                    },
                    "doc": "\tCalculate coverage of intervals from a specific strand.\nWith BED files, requires at least 6 columns (strand is column 6).\n- (STRING): can be + or -\n",
                    "id": "#bedtools-genomecov.cwl/strand"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-trackline"
                    },
                    "doc": "Adds a UCSC/Genome-Browser track line definition in the first line of the output.\n- See here for more details about track line definition:\nhttp://genome.ucsc.edu/goldenPath/help/bedgraph.html\n- NOTE: When adding a trackline definition, the output BedGraph can be easily\nuploaded to the Genome Browser as a custom track,\nBUT CAN NOT be converted into a BigWig file (w/o removing the first line).\n",
                    "id": "#bedtools-genomecov.cwl/trackline"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-trackopts"
                    },
                    "doc": "Writes additional track line definition parameters in the first line.\n- Example:\n-trackopts 'name=\"My Track\" visibility=2 color=255,30,30'\nNote the use of single-quotes if you have spaces in your parameters.\n- (TEXT)\n",
                    "id": "#bedtools-genomecov.cwl/trackopts"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.ibam.path.replace(/^.*[\\\\\\/]/, '').replace(/\\.[^/.]+$/, '') + '.bdg')"
                    },
                    "id": "#bedtools-genomecov.cwl/output_bedfile"
                }
            ],
            "baseCommand": [
                "bedtools",
                "genomecov"
            ],
            "stdout": "$(inputs.ibam.path.replace(/^.*[\\\\\\/]/, '').replace(/\\.[^/.]+$/, '') + '.bdg')",
            "id": "#bedtools-genomecov.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "listing": [
                        "$(inputs.input_file)"
                    ],
                    "class": "InitialWorkDirRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "dukegcb/samtools:1.3",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1
                    },
                    "doc": "Aligned file to be sorted with samtools",
                    "id": "#samtools-index.cwl/input_file"
                }
            ],
            "outputs": [
                {
                    "doc": "Indexed BAM file",
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.input_file.basename)"
                    },
                    "secondaryFiles": ".bai",
                    "id": "#samtools-index.cwl/indexed_file"
                }
            ],
            "baseCommand": [
                "samtools",
                "index"
            ],
            "arguments": [
                {
                    "valueFrom": "$(inputs.input_file.basename + '.bai')",
                    "position": 2
                }
            ],
            "id": "#samtools-index.cwl"
        },
        {
            "class": "CommandLineTool",
            "doc": "Description: filters BAM file(s).\n\n\nUsage: bamtools filter [-in <filename> -in <filename> ... | -list <filelist>] [-out <filename> | [-forceCompression]] [-region <REGION>] [ [-script <filename] | [filterOptions] ]\n\n\nInput & Output:\n\n  -in <BAM filename>                the input BAM file(s) [stdin]\n\n  -list <filename>                  the input BAM file list, one\n\n                                    line per file\n\n  -out <BAM filename>               the output BAM file [stdout]\n\n  -region <REGION>                  only read data from this\n\n                                    genomic region (see documentation for more\n\n                                    details)\n\n  -script <filename>                the filter script file (see\n\n                                    documentation for more details)\n\n  -forceCompression                 if results are sent to stdout\n\n                                    (like when piping to another tool),\n\n                                    default behavior is to leave output\n\n                                    uncompressed. Use this flag to override\n\n                                    and force compression\n\n\n\nGeneral Filters:\n\n  -alignmentFlag <int>              keep reads with this *exact*\n\n                                    alignment flag (for more detailed queries,\n\n                                    see below)\n\n  -insertSize <int>                 keep reads with insert size\n\n                                    that matches pattern\n\n  -mapQuality <[0-255]>             keep reads with map quality\n\n                                    that matches pattern\n\n  -name <string>                    keep reads with name that\n\n                                    matches pattern\n\n  -queryBases <string>              keep reads with motif that\n\n                                    matches pattern\n\n  -tag <TAG:VALUE>                  keep reads with this\n\n                                    key=>value pair\n\n\n\nAlignment Flag Filters:\n\n  -isDuplicate <true/false>         keep only alignments that are\n\n                                    marked as duplicate? [true]\n\n  -isFailedQC <true/false>          keep only alignments that\n\n                                    failed QC? [true]\n\n  -isFirstMate <true/false>         keep only alignments marked as\n\n                                    first mate? [true]\n\n  -isMapped <true/false>            keep only alignments that were\n\n                                    mapped? [true]\n\n  -isMateMapped <true/false>        keep only alignments with\n\n                                    mates that mapped [true]\n\n  -isMateReverseStrand <true/false> keep only alignments with mate\n\n                                    on reverese strand? [true]\n\n  -isPaired <true/false>            keep only alignments that were\n\n                                    sequenced as paired? [true]\n\n  -isPrimaryAlignment <true/false>  keep only alignments marked as\n\n                                    primary? [true]\n\n  -isProperPair <true/false>        keep only alignments that\n\n                                    passed PE resolution? [true]\n\n  -isReverseStrand <true/false>     keep only alignments on\n\n                                    reverse strand? [true]\n\n  -isSecondMate <true/false>        keep only alignments marked as\n\n                                    second mate? [true]\n\n  -isSingleton <true/false>         keep only singletons [true]\n\n\n\nHelp:\n\n  --help, -h                        shows this help text",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ShellCommandRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "reddylab/bamtools:2.2.3",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-alignmentFlag"
                    },
                    "doc": "<int>\nkeep reads with this *exact*\nalignment flag (for more detailed queries, see below)\n",
                    "id": "#bamtools-filter.cwl/alignmentFlag"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-forceCompression"
                    },
                    "doc": "if results are sent to stdout\\n(like when piping to another tool),\ndefault behavior is to leave output\nuncompressed. Use this flag to override\nand force compression.\n",
                    "id": "#bamtools-filter.cwl/forceCompression"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-in"
                    },
                    "doc": "<BAM filename>\nthe input BAM file(s) [stdin if omitted]\n",
                    "id": "#bamtools-filter.cwl/in"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-insertSize"
                    },
                    "doc": "<int>\nkeep reads with insert size that matches pattern\n",
                    "id": "#bamtools-filter.cwl/insertSize"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-isDuplicate"
                    },
                    "doc": "<true/false>\nkeep only alignments that are marked as duplicate? [true]\n",
                    "id": "#bamtools-filter.cwl/isDuplicate"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-isFailedQC"
                    },
                    "doc": "<true/false>          keep only alignments that\nfailed QC? [true]\n",
                    "id": "#bamtools-filter.cwl/isFailedQC"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-isFirstMate"
                    },
                    "doc": "<true/false>         keep only alignments marked as\nfirst mate? [true]\n",
                    "id": "#bamtools-filter.cwl/isFirstMate"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-isMapped"
                    },
                    "doc": "<true/false>            keep only alignments that were\nmapped? [true]\n",
                    "id": "#bamtools-filter.cwl/isMapped"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-isMateMapped"
                    },
                    "doc": "<true/false>        keep only alignments with\nmates that mapped [true]\n",
                    "id": "#bamtools-filter.cwl/isMateMapped"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-isMateReverseStrand"
                    },
                    "doc": "<true/false> keep only alignments with mate\non reverese strand? [true]\n",
                    "id": "#bamtools-filter.cwl/isMateReverseStrand"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-isPaired"
                    },
                    "doc": "<true/false>            keep only alignments that were\nsequenced as paired? [true]\n",
                    "id": "#bamtools-filter.cwl/isPaired"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-isPrimaryAlignment"
                    },
                    "doc": "<true/false>  keep only alignments marked as\nprimary? [true]\n",
                    "id": "#bamtools-filter.cwl/isPrimaryAlignment"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-isProperPair"
                    },
                    "doc": "<true/false>        keep only alignments that\npassed PE resolution? [true]\n",
                    "id": "#bamtools-filter.cwl/isProperPair"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-isReverseStrand"
                    },
                    "doc": "<true/false>     keep only alignments on\nreverse strand? [true]\n",
                    "id": "#bamtools-filter.cwl/isReverseStrand"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-isSecondMate"
                    },
                    "doc": "<true/false>        keep only alignments marked as\nsecond mate? [true]\n",
                    "id": "#bamtools-filter.cwl/isSecondMate"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-isSingleton"
                    },
                    "doc": "<true/false>         keep only singletons [true]\nHelp:\n--help, -h                        shows this help text\n",
                    "id": "#bamtools-filter.cwl/isSingleton"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-mapQuality"
                    },
                    "doc": "<[0-255]>\nkeep reads with map quality that matches pattern (e.g. '>50')\n",
                    "id": "#bamtools-filter.cwl/mapQuality"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-name"
                    },
                    "doc": "<string>\nkeep reads with name that matches pattern\n",
                    "id": "#bamtools-filter.cwl/name"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-out"
                    },
                    "doc": "<BAM filename>\nthe output BAM filename [stdout ]\n",
                    "id": "#bamtools-filter.cwl/out"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-queryBases"
                    },
                    "doc": "<string>\nkeep reads with motif that matches pattern\n",
                    "id": "#bamtools-filter.cwl/queryBases"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-region"
                    },
                    "doc": "<REGION>\nonly read data from this\ngenomic region (see documentation for more\ndetails)\n",
                    "id": "#bamtools-filter.cwl/region"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-script"
                    },
                    "doc": "<filename>\nthe filter script file (see documentation for more details)\n",
                    "id": "#bamtools-filter.cwl/script"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-tag"
                    },
                    "doc": "<TAG:VALUE>\nkeep reads with this key=>value pair\n",
                    "id": "#bamtools-filter.cwl/tag"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.out)"
                    },
                    "id": "#bamtools-filter.cwl/output_file"
                }
            ],
            "baseCommand": [
                "bamtools",
                "filter"
            ],
            "id": "#bamtools-filter.cwl"
        },
        {
            "class": "CommandLineTool",
            "doc": "Tool:   bedGraphToBigWig v 4 - Convert a bedGraph file to bigWig format.",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "dukegcb/bedgraphtobigwig",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1
                    },
                    "doc": "\tbed_graph is a four column file in the format: <chrom> <start> <end> <value>\n",
                    "id": "#bedGraphToBigWig.cwl/bed_graph"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 2
                    },
                    "doc": "\tgenome_sizes is two column: <chromosome name> <size in bases>.\n",
                    "id": "#bedGraphToBigWig.cwl/genome_sizes"
                },
                {
                    "type": "string",
                    "default": ".bw",
                    "id": "#bedGraphToBigWig.cwl/output_suffix"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.bed_graph.path.replace(/^.*[\\\\\\/]/, '').replace(/\\.[^/.]+$/, \"\") + inputs.output_suffix)"
                    },
                    "id": "#bedGraphToBigWig.cwl/output_bigwig"
                }
            ],
            "baseCommand": "bedGraphToBigWig",
            "arguments": [
                {
                    "valueFrom": "$(inputs.bed_graph.path.replace(/^.*[\\\\\\/]/, '').replace(/\\.[^/.]+$/, \"\") + inputs.output_suffix)",
                    "position": 3
                }
            ],
            "id": "#bedGraphToBigWig.cwl"
        },
        {
            "class": "CommandLineTool",
            "doc": "bedSort - Sort a .bed file by chrom,chromStart\nusage:\n   bedSort in.bed out.bed\nin.bed and out.bed may be the same.\n",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "dleehr/docker-hubutils",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1
                    },
                    "doc": "Bed or bedGraph file to be sorted",
                    "id": "#bedSort.cwl/bed_file"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.bed_file.path.replace(/^.*[\\\\\\/]/, '') + \"_sorted\")"
                    },
                    "id": "#bedSort.cwl/bed_file_sorted"
                }
            ],
            "baseCommand": "bedSort",
            "arguments": [
                {
                    "valueFrom": "$(inputs.bed_file.path.replace(/^.*[\\\\\\/]/, '') + \"_sorted\")",
                    "position": 2
                }
            ],
            "id": "#bedSort.cwl"
        },
        {
            "class": "CommandLineTool",
            "doc": "bigWigToBedGraph - Convert from bigWig to bedGraph format.\nusage:\n   bigWigToBedGraph in.bigWig out.bedGraph\noptions:\n   -chrom=chr1 - if set restrict output to given chromosome\n   -start=N - if set, restrict output to only that over start\n   -end=N - if set, restict output to only that under end\n   -udcDir=/dir/to/cache - place to put cache for remote bigBed/bigWigs\n",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "dleehr/docker-hubutils",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1
                    },
                    "doc": "Bed file to be clipped",
                    "id": "#bigWigToBedGraph.cwl/bigwig_file"
                },
                {
                    "type": "string",
                    "default": ".bdg",
                    "id": "#bigWigToBedGraph.cwl/output_suffix"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.bigwig_file.path.replace(/^.*[\\\\\\/]/, '').replace(/\\.[^/.]+$/, \"\") + inputs.output_suffix)"
                    },
                    "id": "#bigWigToBedGraph.cwl/output_bedgraph"
                }
            ],
            "baseCommand": "bigWigToBedGraph",
            "arguments": [
                {
                    "valueFrom": "$(inputs.bigwig_file.path.replace(/^.*[\\\\\\/]/, '').replace(/\\.[^/.]+$/, \"\") + inputs.output_suffix)",
                    "position": 2
                }
            ],
            "id": "#bigWigToBedGraph.cwl"
        },
        {
            "class": "CommandLineTool",
            "doc": "usage: An example usage is:$ bamCoverage -b reads.bam -o coverage.bw\n\nThis tool takes an alignment of reads or fragments as input (BAM file) and\ngenerates a coverage track (bigWig or bedGraph) as output. The coverage is\ncalculated as the number of reads per bin, where bins are short consecutive\ncounting windows of a defined size. It is possible to extended the length of\nthe reads to better reflect the actual fragment length. *bamCoverage* offers\nnormalization by scaling factor, Reads Per Kilobase per Million mapped reads\n(RPKM), and 1x depth (reads per genome coverage, RPGC).\nRequired arguments:\n  --bam BAM file, -b BAM file\n                        BAM file to process (default: None)\nOutput:\n  --outFileName FILENAME, -o FILENAME\n                        Output file name. (default: None)\n  --outFileFormat {bigwig,bedgraph}, -of {bigwig,bedgraph}\n                        Output file type. Either \"bigwig\" or \"bedgraph\".\n                        (default: bigwig)\nOptional arguments:\n  --help, -h            show this help message and exit\n  --scaleFactor SCALEFACTOR\n                        The smooth length defines a window, larger than the\n                        binSize, to average the number of reads. For example,\n                        if the \u2013binSize is set to 20 and the \u2013smoothLength is\n                        set to 60, then, for each bin, the average of the bin\n                        and its left and right neighbors is considered.\n                        Any value smaller than \u2013binSize will be ignored and\n                        no smoothing will be applied. (default: 1.0)\n  --MNase               Determine nucleosome positions from MNase-seq data.\n                        Only 3 nucleotides at the center of each fragment are\n                        counted. The fragment ends are defined by the two mate\n                        reads. Only fragment lengthsbetween 130 - 200 bp are\n                        considered to avoid dinucleosomes or other\n                        artifacts.*NOTE*: Requires paired-end data. A bin size\n                        of 1 is recommended. (default: False)\n  --filterRNAstrand {forward,reverse}\n                        Selects RNA-seq reads (single-end or paired-end) in\n                        the given strand. (default: None)\n  --version             show program's version number and exit\n  --binSize INT bp, -bs INT bp\n                        Size of the bins, in bases, for the output of the\n                        bigwig/bedgraph file. (default: 50)\n  --region CHR:START:END, -r CHR:START:END\n                        Region of the genome to limit the operation to - this\n                        is useful when testing parameters to reduce the\n                        computing time. The format is chr:start:end, for\n                        example --region chr10 or --region\n                        chr10:456700:891000. (default: None)\n  --blackListFileName BED file, -bl BED file\n                        A BED file containing regions that should be excluded\n                        from all analyses. Currently this works by rejecting\n                        genomic chunks that happen to overlap an entry.\n                        Consequently, for BAM files, if a read partially\n                        overlaps a blacklisted region or a fragment spans over\n                        it, then the read/fragment might still be considered.\n                        (default: None)\n  --numberOfProcessors INT, -p INT\n                        Number of processors to use. Type \"max/2\" to use half\n                        the maximum number of processors or \"max\" to use all\n                        available processors. (default: max/2)\n  --verbose, -v         Set to see processing messages. (default: False)\nRead coverage normalization options:\n  --normalizeTo1x EFFECTIVE GENOME SIZE LENGTH\n                        Report read coverage normalized to 1x sequencing depth\n                        (also known as Reads Per Genomic Content (RPGC)).\n                        Sequencing depth is defined as: (total number of\n                        mapped reads * fragment length) / effective genome\n                        size. The scaling factor used is the inverse of the\n                        sequencing depth computed for the sample to match the\n                        1x coverage. To use this option, the effective genome\n                        size has to be indicated after the option. The\n                        effective genome size is the portion of the genome\n                        that is mappable. Large fractions of the genome are\n                        stretches of NNNN that should be discarded. Also, if\n                        repetitive regions were not included in the mapping of\n                        reads, the effective genome size needs to be adjusted\n                        accordingly. Common values are: mm9: 2,150,570,000;\n                        hg19:2,451,960,000; dm3:121,400,000 and\n                        ce10:93,260,000. See Table 2 of http://www.plosone.org\n                        /article/info:doi/10.1371/journal.pone.0030377 or http\n                        ://www.nature.com/nbt/journal/v27/n1/fig_tab/nbt.1518_\n                        T1.html for several effective genome sizes. (default:\n                        None)\n--ignoreForNormalization IGNOREFORNORMALIZATION [IGNOREFORNORMALIZATION ...]\n                        A list of space-delimited chromosome names containing\n                        those chromosomes that should be excluded for\n                        computing the normalization. This is useful when\n                        considering samples with unequal coverage across\n                        chromosomes, like male samples. An usage examples is\n                        --ignoreForNormalization chrX chrM. (default: None)\n  --skipNonCoveredRegions, --skipNAs\n                        This parameter determines if non-covered regions\n                        (regions without overlapping reads) in a BAM file\n                        should be skipped. The default is to treat those\n                        regions as having a value of zero. The decision to\n                        skip non-covered regions depends on the interpretation\n                        of the data. Non-covered regions may represent, for\n                        example, repetitive regions that should be skipped.\n                        (default: False)\n  --smoothLength INT bp\n                        The smooth length defines a window, larger than the\n                        binSize, to average the number of reads. For example,\n                        if the --binSize is set to 20 and the --smoothLength\n                        is set to 60, then, for each bin, the average of the\n                        bin and its left and right neighbors is considered.\n                        Any value smaller than --binSize will be ignored and\n                        no smoothing will be applied. (default: None)\nRead processing options:\n  --extendReads [INT bp], -e [INT bp]\n                        This parameter allows the extension of reads to\n                        fragment size. If set, each read is extended, without\n                        exception. *NOTE*: This feature is generally NOT\n                        recommended for spliced-read data, such as RNA-seq, as\n                        it would extend reads over skipped regions. *Single-\n                        end*: Requires a user specified value for the final\n                        fragment length. Reads that already exceed this\n                        fragment length will not be extended. *Paired-end*:\n                        Reads with mates are always extended to match the\n                        fragment size defined by the two read mates. Unmated\n                        reads, mate reads that map too far apart (>4x fragment\n                        length) or even map to different chromosomes are\n                        treated like single-end reads. The input of a fragment\n                        length value is optional. If no value is specified, it\n                        is estimated from the data (mean of the fragment size\n                        of all mate reads). (default: False)\n  --ignoreDuplicates    If set, reads that have the same orientation and start\n                        position will be considered only once. If reads are\n                        paired, the mate's position also has to coincide to\n                        ignore a read. (default: False)\n  --minMappingQuality INT\n                        If set, only reads that have a mapping quality score\n                        of at least this are considered. (default: None)\n  --centerReads         By adding this option, reads are centered with respect\n                        to the fragment length. For paired-end data, the read\n                        is centered at the fragment length defined by the two\n                        ends of the fragment. For single-end data, the given\n                        fragment length is used. This option is useful to get\n                        a sharper signal around enriched regions. (default:\n                        False)\n  --samFlagInclude INT  Include reads based on the SAM flag. For example, to\n                        get only reads that are the first mate, use a flag of\n                        64. This is useful to count properly paired reads only\n                        once, as otherwise the second mate will be also\n                        considered for the coverage. (default: None)\n  --samFlagExclude INT  Exclude reads based on the SAM flag. For example, to\n                        get only reads that map to the forward strand, use\n                        --samFlagExclude 16, where 16 is the SAM flag for\n                        reads that map to the reverse strand. (default: None)\n",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "reddylab/deeptools:3.0.1",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--MNase"
                    },
                    "doc": "Determine nucleosome positions from MNase-seq data.\nOnly 3 nucleotides at the center of each fragment are\ncounted. The fragment ends are defined by the two mate\nreads. Only fragment lengthsbetween 130 - 200 bp are\nconsidered to avoid dinucleosomes or other\nartifacts.*NOTE*: Requires paired-end data. A bin size\nof 1 is recommended. (default: False)\n",
                    "id": "#deeptools-bamcoverage.cwl/MNase"
                },
                {
                    "type": "File",
                    "secondaryFiles": [
                        ".bai"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bam"
                    },
                    "doc": "BAM file to process ",
                    "id": "#deeptools-bamcoverage.cwl/bam"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--binSize"
                    },
                    "doc": "INT bp\nSize of the bins, in bases, for the output of the\nbigwig/bedgraph file. (default: 50)\n",
                    "id": "#deeptools-bamcoverage.cwl/binSize"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--blackListFileName"
                    },
                    "doc": "BED file\nA BED file containing regions that should be excluded\nfrom all analyses. Currently this works by rejecting\ngenomic chunks that happen to overlap an entry.\nConsequently, for BAM files, if a read partially\noverlaps a blacklisted region or a fragment spans over\nit, then the read/fragment might still be considered.\n(default: None)\n",
                    "id": "#deeptools-bamcoverage.cwl/blackListFileName"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--centerReads"
                    },
                    "doc": "By adding this option, reads are centered with respect\nto the fragment length. For paired-end data, the read\nis centered at the fragment length defined by the two\nends of the fragment. For single-end data, the given\nfragment length is used. This option is useful to get\na sharper signal around enriched regions. (default:\nFalse)\n",
                    "id": "#deeptools-bamcoverage.cwl/centerReads"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--extendReads"
                    },
                    "doc": "INT bp\nThis parameter allows the extension of reads to\nfragment size. If set, each read is extended, without\nexception. *NOTE*: This feature is generally NOT\nrecommended for spliced-read data, such as RNA-seq, as\nit would extend reads over skipped regions. *Single-\nend*: Requires a user specified value for the final\nfragment length. Reads that already exceed this\nfragment length will not be extended. *Paired-end*:\nReads with mates are always extended to match the\nfragment size defined by the two read mates. Unmated\nreads, mate reads that map too far apart (>4x fragment\nlength) or even map to different chromosomes are\ntreated like single-end reads. The input of a fragment\nlength value is optional. If no value is specified, it\nis estimated from the data (mean of the fragment size\nof all mate reads). (default: False)\n",
                    "id": "#deeptools-bamcoverage.cwl/extendReads"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--filterRNAstrand"
                    },
                    "doc": "{forward,reverse}\nSelects RNA-seq reads (single-end or paired-end) in\nthe given strand. (default: None)\n",
                    "id": "#deeptools-bamcoverage.cwl/filterRNAstrand"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--ignoreDuplicates"
                    },
                    "doc": "If set, reads that have the same orientation and start\nposition will be considered only once. If reads are\npaired, the mate's position also has to coincide to\nignore a read. (default: False)\n",
                    "id": "#deeptools-bamcoverage.cwl/ignoreDuplicates"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "string"
                        }
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--ignoreForNormalization"
                    },
                    "doc": "A list of space-delimited chromosome names containing those chromosomes\nthat should be excluded for computing the normalization. This is useful\nwhen considering samples with unequal coverage across chromosomes, like\nmale samples. An usage examples is \u2013ignoreForNormalization chrX chrM.\n",
                    "id": "#deeptools-bamcoverage.cwl/ignoreForNormalization"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--minMappingQuality"
                    },
                    "doc": "INT\nIf set, only reads that have a mapping quality score\nof at least this are considered. (default: None)\n",
                    "id": "#deeptools-bamcoverage.cwl/minMappingQuality"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--normalizeUsing"
                    },
                    "doc": "Possible choices: RPKM, CPM, BPM, RPGC\n\nUse one of the entered methods to normalize the number of reads per bin.\nBy default, no normalization is performed. RPKM = Reads Per Kilobase per\nMillion mapped reads; CPM = Counts Per Million mapped reads, same as\nCPM in RNA-seq; BPM = Bins Per Million mapped reads, same as TPM in\nRNA-seq; RPGC = reads per genomic content (1x normalization);\nMapped reads are considered after blacklist filtering (if applied).\nRPKM (per bin) = number of reads per bin / (number of mapped reads (in millions) * bin length (kb)).\nCPM (per bin) = number of reads per bin / number of mapped reads (in millions).\nBPM (per bin) = number of reads per bin / sum of all reads per bin (in millions).\nRPGC (per bin) = number of reads per bin / scaling factor for 1x average coverage.\nThis scaling factor, in turn, is determined from the sequencing depth:\n(total number of mapped reads * fragment length) / effective genome size.\nThe scaling factor used is the inverse of the sequencing depth\ncomputed for the sample to match the 1x coverage.\nThis option requires \u2013effectiveGenomeSize.\nEach read is considered independently, if you want to only count one\nmate from a pair in paired-end data, then use\nthe \u2013samFlagInclude/\u2013samFlagExclude options.\n",
                    "id": "#deeptools-bamcoverage.cwl/normalizeUsing"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--numberOfProcessors"
                    },
                    "doc": "INT\nNumber of processors to use. Type \"max/2\" to use half\nthe maximum number of processors or \"max\" to use all\navailable processors. (default: max/2)\n",
                    "id": "#deeptools-bamcoverage.cwl/numberOfProcessors"
                },
                {
                    "type": "string",
                    "default": "bigwig",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFileFormat"
                    },
                    "doc": "{bigwig,bedgraph}, -of {bigwig,bedgraph}\nOutput file type. Either \"bigwig\" or \"bedgraph\".\n(default: bigwig)\n",
                    "id": "#deeptools-bamcoverage.cwl/outFileFormat"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "FILENAME\nOutput file name. (default: input BAM filename with bigwig [*.bw] or bedgraph [*.bdg] extension.)\n",
                    "id": "#deeptools-bamcoverage.cwl/outFileName"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Suffix used for output file (input BAM filename + suffix)",
                    "id": "#deeptools-bamcoverage.cwl/output_suffix"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--region"
                    },
                    "doc": "CHR:START:END\nRegion of the genome to limit the operation to - this\nis useful when testing parameters to reduce the\ncomputing time. The format is chr:start:end, for\nexample --region chr10 or --region\nchr10:456700:891000. (default: None)\n",
                    "id": "#deeptools-bamcoverage.cwl/region"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--samFlagExclude"
                    },
                    "doc": "INT  \nExclude reads based on the SAM flag. For example, to\nget only reads that map to the forward strand, use\n--samFlagExclude 16, where 16 is the SAM flag for\nreads that map to the reverse strand. (default: None)\n",
                    "id": "#deeptools-bamcoverage.cwl/samFlagExclude"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--samFlagInclude"
                    },
                    "doc": "INT  \nInclude reads based on the SAM flag. For example, to\nget only reads that are the first mate, use a flag of\n64. This is useful to count properly paired reads only\nonce, as otherwise the second mate will be also\nconsidered for the coverage. (default: None)\n",
                    "id": "#deeptools-bamcoverage.cwl/samFlagInclude"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scaleFactor"
                    },
                    "doc": "The smooth length defines a window, larger than the\nbinSize, to average the number of reads. For example,\nif the \u2013binSize is set to 20 and the \u2013smoothLength is\nset to 60, then, for each bin, the average of the bin\nand its left and right neighbors is considered.\nAny value smaller than \u2013binSize will be ignored and\nno smoothing will be applied. (default: 1.0)\n",
                    "id": "#deeptools-bamcoverage.cwl/scaleFactor"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--skipNonCoveredRegions"
                    },
                    "doc": "--skipNonCoveredRegions, --skipNAs\nThis parameter determines if non-covered regions\n(regions without overlapping reads) in a BAM file\nshould be skipped. The default is to treat those\nregions as having a value of zero. The decision to\nskip non-covered regions depends on the interpretation\nof the data. Non-covered regions may represent, for\nexample, repetitive regions that should be skipped.\n(default: False)\n",
                    "id": "#deeptools-bamcoverage.cwl/skipNonCoveredRegions"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--smoothLength"
                    },
                    "doc": "INT bp\nThe smooth length defines a window, larger than the\nbinSize, to average the number of reads. For example,\nif the --binSize is set to 20 and the --smoothLength\nis set to 60, then, for each bin, the average of the\nbin and its left and right neighbors is considered.\nAny value smaller than --binSize will be ignored and\nno smoothing will be applied. (default: None)\nRead processing options:\n",
                    "id": "#deeptools-bamcoverage.cwl/smoothLength"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--verbose"
                    },
                    "doc": "--verbose         \nSet to see processing messages. (default: False)\n",
                    "id": "#deeptools-bamcoverage.cwl/verbose"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--version"
                    },
                    "doc": "show program's version number and exit",
                    "id": "#deeptools-bamcoverage.cwl/version"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${ if (inputs.outFileName) return inputs.outFileName; if (inputs.output_suffix) return inputs.bam.path.replace(/^.*[\\\\\\/]/, \"\").replace(/\\.[^/.]+$/, \"\") + inputs.output_suffix; if (inputs.outFileFormat == \"bedgraph\") return inputs.bam.path.replace(/^.*[\\\\\\/]/, \"\").replace(/\\.[^/.]+$/, \"\") + \".bdg\"; return inputs.bam.path.replace(/^.*[\\\\\\/]/, \"\").replace(/\\.[^/.]+$/, \"\") + \".bw\"; }"
                    },
                    "id": "#deeptools-bamcoverage.cwl/output_bam_coverage"
                }
            ],
            "baseCommand": "bamCoverage",
            "arguments": [
                {
                    "valueFrom": "${ if (inputs.outFileName) return inputs.outFileName; if (inputs.output_suffix) return inputs.bam.path.replace(/^.*[\\\\\\/]/, \"\").replace(/\\.[^/.]+$/, \"\") + inputs.output_suffix; if (inputs.outFileFormat == \"bedgraph\") return inputs.bam.path.replace(/^.*[\\\\\\/]/, \"\").replace(/\\.[^/.]+$/, \"\") + \".bdg\"; return inputs.bam.path.replace(/^.*[\\\\\\/]/, \"\").replace(/\\.[^/.]+$/, \"\") + \".bw\"; }",
                    "prefix": "--outFileName",
                    "position": 3
                }
            ],
            "id": "#deeptools-bamcoverage.cwl"
        },
        {
            "class": "CommandLineTool",
            "doc": "Negate minus strand bedGraph values.",
            "hints": [
                {
                    "dockerPull": "reddylab/workflow-utils:ggr",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1
                    },
                    "id": "#negate-minus-strand-bedgraph.cwl/bedgraph_file"
                },
                {
                    "type": "string",
                    "id": "#negate-minus-strand-bedgraph.cwl/output_filename"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_filename)"
                    },
                    "id": "#negate-minus-strand-bedgraph.cwl/negated_minus_bdg"
                }
            ],
            "baseCommand": [
                "awk",
                "BEGIN{OFS=\"\\t\";FS=\"\\t\"} {if ($4 > 0) {print $1,$2,$3,-$4} else {print $0}}"
            ],
            "stdout": "$(inputs.output_filename)",
            "id": "#negate-minus-strand-bedgraph.cwl"
        },
        {
            "class": "CommandLineTool",
            "doc": "In its default mode, this program aligns input reads against a reference\ntranscriptome with Bowtie and calculates expression values using the\nalignments. RSEM assumes the data are single-end reads with quality\nscores, unless the '--paired-end' or '--no-qualities' options are\nspecified. Users may use an alternative aligner by specifying one of the\n--sam and --bam options, and providing an alignment file in the\nspecified format. However, users should make sure that they align\nagainst the indices generated by 'rsem-prepare-reference' and the\nalignment file satisfies the requirements mentioned in ARGUMENTS\nsection.\nOne simple way to make the alignment file satisfying RSEM's requirements\n(assuming the aligner used put mates in a paired-end read adjacent) is\nto use 'convert-sam-for-rsem' script. This script only accept SAM format\nfiles as input. If a BAM format file is obtained, please use samtools to\nconvert it to a SAM file first. For example, if '/ref/mouse_125' is the\n'reference_name' and the SAM file is named 'input.sam', you can run the\nfollowing command:\n  convert-sam-for-rsem /ref/mouse_125 input.sam -o input_for_rsem.sam\nFor details, please refer to 'convert-sam-for-rsem's documentation page.\nThe SAM/BAM format RSEM uses is v1.4. However, it is compatible with old\nSAM/BAM format. However, RSEM cannot recognize 0x100 in the FLAG field.\nIn addition, RSEM requires SEQ and QUAL are not '*'.\nThe user must run 'rsem-prepare-reference' with the appropriate\nreference before using this program.\nFor single-end data, it is strongly recommended that the user provide\nthe fragment length distribution parameters (--fragment-length-mean and\n--fragment-length-sd). For paired-end data, RSEM will automatically\nlearn a fragment length distribution from the data.\nPlease note that some of the default values for the Bowtie parameters\nare not the same as those defined for Bowtie itself.\nThe temporary directory and all intermediate files will be removed when\nRSEM finishes unless '--keep-intermediate-files' is specified.\nWith the '--calc-pme' option, posterior mean estimates will be\ncalculated in addition to maximum likelihood estimates.\nWith the '--calc-ci' option, 95% credibility intervals and posterior\nmean estimates will be calculated in addition to maximum likelihood\nestimates.\n",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "reddylab/rsem:1.2.25",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 2,
                        "prefix": "--bam"
                    },
                    "doc": "BAM formatted input file. If \"-\" is specified for the filename,\nBAM input is instead assumed to come from standard input. RSEM\nrequires all alignments of the same read group together. For\npaired-end reads, RSEM also requires the two mates of any alignment\nbe adjacent. See Description section for how to make input file obey\nRSEM's requirements.\n",
                    "id": "#rsem-calculate-expression.cwl/bam"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bowtie-chunkmbs"
                    },
                    "doc": "<int>\n(Bowtie parameter) memory allocated for best first alignment\ncalculation (Default: 0 - use Bowtie's default)\n",
                    "id": "#rsem-calculate-expression.cwl/bowtie-chunkmbs"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bowtie-e"
                    },
                    "doc": "<int>\n(Bowtie parameter) max sum of mismatch quality scores across the\nalignment. (Default: 99999999)\n",
                    "id": "#rsem-calculate-expression.cwl/bowtie-e"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bowtie-m"
                    },
                    "doc": "<int>\n(Bowtie parameter) suppress all alignments for a read if > <int>\nvalid alignments exist. (Default: 200)\n",
                    "id": "#rsem-calculate-expression.cwl/bowtie-m"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bowtie-n"
                    },
                    "doc": "<int>\n(Bowtie parameter) max # of mismatches in the seed. (Range: 0-3,\nDefault: 2)\n",
                    "id": "#rsem-calculate-expression.cwl/bowtie-n"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bowtie-path"
                    },
                    "doc": "<path>\nThe path to the Bowtie executables. (Default: the path to the Bowtie\nexecutables is assumed to be in the user's PATH environment\nvariable)\n",
                    "id": "#rsem-calculate-expression.cwl/bowtie-path"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bowtie2"
                    },
                    "doc": "Use Bowtie 2 instead of Bowtie to align reads. Since currently RSEM\ndoes not handle indel, local and discordant alignments, the Bowtie2\nparameters are set in a way to avoid those alignments. In\nparticular, we use options '--sensitive --dpad 0 --gbar 99999999\n--mp 1,1 --np 1 --score-min L,0,-0.1' by default. The last parameter\nof '--score-min', '-0.1', is the negative of maximum mismatch rate.\nThis rate can be set by option '--bowtie2-mismatch-rate'. If reads\nare paired-end, we additionally use options '--no-mixed' and\n'--no-discordant'. (Default: off)\n",
                    "id": "#rsem-calculate-expression.cwl/bowtie2"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bowtie2-k"
                    },
                    "doc": "<int>\n(Bowtie 2 parameter) Find up to <int> alignments per read. (Default:\n200)\n",
                    "id": "#rsem-calculate-expression.cwl/bowtie2-k"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bowtie2-mismatch-rate"
                    },
                    "doc": "<double>\n(Bowtie 2 parameter) The maximum mismatch rate allowed. (Default:\n0.1)\n",
                    "id": "#rsem-calculate-expression.cwl/bowtie2-mismatch-rate"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bowtie2-path"
                    },
                    "doc": "<path>\n(Bowtie 2 parameter) The path to the Bowtie 2 executables. (Default:\nthe path to the Bowtie 2 executables is assumed to be in the user's\nPATH environment variable)\n",
                    "id": "#rsem-calculate-expression.cwl/bowtie2-path"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bowtie2-sensitivity-level"
                    },
                    "doc": "<string>\n(Bowtie 2 parameter) Set Bowtie 2's preset options in --end-to-end\nmode. This option controls how hard Bowtie 2 tries to find\nalignments. <string> must be one of \"very_fast\", \"fast\", \"sensitive\"\nand \"very_sensitive\". The four candidates correspond to Bowtie 2's\n\"--very-fast\", \"--fast\", \"--sensitive\" and \"--very-sensitive\"\noptions. (Default: \"sensitive\" - use Bowtie 2's default)\n",
                    "id": "#rsem-calculate-expression.cwl/bowtie2-sensitivity-level"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--calc-ci"
                    },
                    "doc": "--calc-ci\nCalculate 95% credibility intervals and posterior mean estimates.\nThe credibility level can be changed by setting\n'--ci-credibility-level'. (Default: off)\n",
                    "id": "#rsem-calculate-expression.cwl/calc-ci"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--calc-pme"
                    },
                    "doc": "--calc-pme\nRun RSEM's collapsed Gibbs sampler to calculate posterior mean\nestimates. (Default: off)\n",
                    "id": "#rsem-calculate-expression.cwl/calc-pme"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--ci-credibility-level"
                    },
                    "doc": "<double>\nThe credibility level for credibility intervals. (Default: 0.95)\n",
                    "id": "#rsem-calculate-expression.cwl/ci-credibility-level"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--ci-memory"
                    },
                    "doc": "<int>\nMaximum size (in memory, MB) of the auxiliary buffer used for\ncomputing credibility intervals (CI). Set it larger for a faster CI\ncalculation. However, leaving 2 GB memory free for other usage is\nrecommended. (Default: 1024)\n",
                    "id": "#rsem-calculate-expression.cwl/ci-memory"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--ci-number-of-samples-per-count-vector"
                    },
                    "doc": "<int>\nThe number of read generating probability vectors sampled per\nsampled count vector. The crebility intervals are calculated by\nfirst sampling P(C | D) and then sampling P(Theta | C) for each\nsampled count vector. This option controls how many Theta vectors\nare sampled per sampled count vector. (Default: 50)\n",
                    "id": "#rsem-calculate-expression.cwl/ci-number-of-samples-per-count-vector"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "position": 3,
                        "itemSeparator": ","
                    },
                    "doc": "Comma-separated list of files containing downstream reads which are\npaired with the upstream reads. By default, these files are assumed\nto be in FASTQ format. If the --no-qualities option is specified,\nthen FASTA format is expected.\n",
                    "id": "#rsem-calculate-expression.cwl/downstream_read_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--estimate-rspd"
                    },
                    "doc": "Set this option if you want to estimate the read start position\ndistribution (RSPD) from data. Otherwise, RSEM will use a uniform\nRSPD. (Default: off)\n",
                    "id": "#rsem-calculate-expression.cwl/estimate-rspd"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--forward-prob"
                    },
                    "doc": "<double>\nProbability of generating a read from the forward strand of a\ntranscript. Set to 1 for a strand-specific protocol where all\n(upstream) reads are derived from the forward strand, 0 for a\nstrand-specific protocol where all (upstream) read are derived from\nthe reverse strand, or 0.5 for a non-strand-specific protocol.\n(Default: 0.5)\n",
                    "id": "#rsem-calculate-expression.cwl/forward-prob"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--fragment-length-max"
                    },
                    "doc": "<int>\nMaximum read/insert length allowed. This is also the value for the\nBowtie/Bowtie 2 -X option. (Default: 1000)\n",
                    "id": "#rsem-calculate-expression.cwl/fragment-length-max"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--fragment-length-mean"
                    },
                    "doc": "<double>\n(single-end data only) The mean of the fragment length distribution,\nwhich is assumed to be a Gaussian. (Default: -1, which disables use\nof the fragment length distribution)\n",
                    "id": "#rsem-calculate-expression.cwl/fragment-length-mean"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--fragment-length-min"
                    },
                    "doc": "<int>\nMinimum read/insert length allowed. This is also the value for the\nBowtie/Bowtie2 -I option. (Default: 1)\n",
                    "id": "#rsem-calculate-expression.cwl/fragment-length-min"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--fragment-length-sd"
                    },
                    "doc": "<double>\n(single-end data only) The standard deviation of the fragment length\ndistribution, which is assumed to be a Gaussian. (Default: 0, which\nassumes that all fragments are of the same length, given by the\nrounded value of --fragment-length-mean)\n",
                    "id": "#rsem-calculate-expression.cwl/fragment-length-sd"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--gibbs-burnin"
                    },
                    "doc": "<int>\nThe number of burn-in rounds for RSEM's Gibbs sampler. Each round\npasses over the entire data set once. If RSEM can use multiple\nthreads, multiple Gibbs samplers will start at the same time and all\nsamplers share the same burn-in number. (Default: 200)\n",
                    "id": "#rsem-calculate-expression.cwl/gibbs-burnin"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--gibbs-number-of-samples"
                    },
                    "doc": "<int>\nThe total number of count vectors RSEM will collect from its Gibbs\nsamplers. (Default: 1000)\n",
                    "id": "#rsem-calculate-expression.cwl/gibbs-number-of-samples"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--gibbs-sampling-gap"
                    },
                    "doc": "<int>\nThe number of rounds between two succinct count vectors RSEM\ncollects. If the count vector after round N is collected, the count\nvector after round N + <int> will also be collected. (Default: 1)\n",
                    "id": "#rsem-calculate-expression.cwl/gibbs-sampling-gap"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--keep-intermediate-files"
                    },
                    "doc": "Keep temporary files generated by RSEM. RSEM creates a temporary\ndirectory, 'sample_name.temp', into which it puts all intermediate\noutput files. If this directory already exists, RSEM overwrites all\nfiles generated by previous RSEM runs inside of it. By default,\nafter RSEM finishes, the temporary directory is deleted. Set this\noption to prevent the deletion of this directory and the\nintermediate files inside of it. (Default: off)\n",
                    "id": "#rsem-calculate-expression.cwl/keep-intermediate-files"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--no-bam-output"
                    },
                    "doc": "Do not output any BAM file. (Default: off)",
                    "id": "#rsem-calculate-expression.cwl/no-bam-output"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--no-qualities"
                    },
                    "doc": "Input reads do not contain quality scores. (Default: off)",
                    "id": "#rsem-calculate-expression.cwl/no-qualities"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--num-rspd-bins"
                    },
                    "doc": "<int>\nNumber of bins in the RSPD. Only relevant when '--estimate-rspd' is\nspecified. Use of the default setting is recommended. (Default: 20)\n",
                    "id": "#rsem-calculate-expression.cwl/num-rspd-bins"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--num-threads"
                    },
                    "doc": "--num-threads <int>\nNumber of threads to use. Both Bowtie/Bowtie2, expression estimation\nand 'samtools sort' will use this many threads. (Default: 1)\n",
                    "id": "#rsem-calculate-expression.cwl/num-threads"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--output-genome-bam"
                    },
                    "doc": "--output-genome-bam\nGenerate a BAM file, 'sample_name.genome.bam', with alignments\nmapped to genomic coordinates and annotated with their posterior\nprobabilities. In addition, RSEM will call samtools (included in\nRSEM package) to sort and index the bam file.\n'sample_name.genome.sorted.bam' and\n'sample_name.genome.sorted.bam.bai' will be generated. (Default:\noff)\n",
                    "id": "#rsem-calculate-expression.cwl/output-genome-bam"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--paired-end"
                    },
                    "doc": "Input reads are paired-end reads. (Default: off)",
                    "id": "#rsem-calculate-expression.cwl/paired-end"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--phred33-quals"
                    },
                    "doc": "Input quality scores are encoded as Phred+33. (Default: on)\n",
                    "id": "#rsem-calculate-expression.cwl/phred33-quals"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--phred64-quals"
                    },
                    "doc": "Input quality scores are encoded as Phred+64 (default for GA\nPipeline ver. >= 1.3). (Default: off)\n",
                    "id": "#rsem-calculate-expression.cwl/phred64-quals"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--quiet"
                    },
                    "doc": "--quiet\nSuppress the output of logging information. (Default: off)\n",
                    "id": "#rsem-calculate-expression.cwl/quiet"
                },
                {
                    "type": "Directory",
                    "doc": "Directory containing <reference_name>.seq, <reference_name>.transcripts.fa, ...\nThe user must have run 'rsem-prepare-reference' with a reference_name\nbefore running this program.\n",
                    "id": "#rsem-calculate-expression.cwl/reference_files"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 4,
                        "valueFrom": "$(inputs.reference_files.path + \"/\" + self)"
                    },
                    "doc": "The name of the reference used. The user must have run\n'rsem-prepare-reference' with this reference_name before running\nthis program.\n",
                    "id": "#rsem-calculate-expression.cwl/reference_name"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 2,
                        "prefix": "--sam"
                    },
                    "doc": "SAM formatted input file. If \"-\" is specified for the filename,\nSAM input is instead assumed to come from standard input. RSEM\nrequires all alignments of the same read group together. For\npaired-end reads, RSEM also requires the two mates of any alignment\nbe adjacent. See Description section for how to make input file obey\nRSEM's requirements.\n",
                    "id": "#rsem-calculate-expression.cwl/sam"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sam-header-info"
                    },
                    "doc": "<file>\nRSEM reads header information from input by default. If this option\nis on, header information is read from the specified file. For the\nformat of the file, please see SAM official website. (Default: \"\")\n",
                    "id": "#rsem-calculate-expression.cwl/sam-header-info"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 5
                    },
                    "doc": "The name of the sample analyzed. All output files are prefixed by\nthis name (e.g., sample_name.genes.results)\n",
                    "id": "#rsem-calculate-expression.cwl/sample_name"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sampling-for-bam"
                    },
                    "doc": "--sampling-for-bam\nWhen RSEM generates a BAM file, instead of outputing all alignments\na read has with their posterior probabilities, one alignment is\nsampled according to the posterior probabilities. The sampling\nprocedure includes the alignment to the \"noise\" transcript, which\ndoes not appear in the BAM file. Only the sampled alignment has a\nweight of 1. All other alignments have weight 0. If the \"noise\"\ntranscript is sampled, all alignments appeared in the BAM file\nshould have weight 0. (Default: off)\n",
                    "id": "#rsem-calculate-expression.cwl/sampling-for-bam"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--samtools-sort-mem"
                    },
                    "doc": "<string>\nSet the maximum memory per thread that can be used by 'samtools\nsort'. <string> represents the memory and accepts suffices 'K/M/G'.\nRSEM will pass <string> to the '-m' option of 'samtools sort'.\nPlease note that the default used here is different from the default\nused by samtools. (Default: 1G)\n",
                    "id": "#rsem-calculate-expression.cwl/samtools-sort-mem"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seed"
                    },
                    "doc": "<int>\nSet the seed for the random number generators used in calculating\nposterior mean estimates and credibility intervals. The seed must be\na non-negative 32 bit interger. (Default: off)\n",
                    "id": "#rsem-calculate-expression.cwl/seed"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seed-length"
                    },
                    "doc": "<int>\nSeed length used by the read aligner. Providing the correct value is\nimportant for RSEM. If RSEM runs Bowtie, it uses this value for\nBowtie's seed length parameter. Any read with its or at least one of\nits mates' (for paired-end reads) length less than this value will\nbe ignored. If the references are not added poly(A) tails, the\nminimum allowed value is 5, otherwise, the minimum allowed value is\n25. Note that this script will only check if the value >= 5 and give\na warning message if the value < 25 but >= 5. (Default: 25)\n",
                    "id": "#rsem-calculate-expression.cwl/seed-length"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--solexa-quals"
                    },
                    "doc": "Input quality scores are solexa encoded (from GA Pipeline ver. <\n1.3). (Default: off)\n",
                    "id": "#rsem-calculate-expression.cwl/solexa-quals"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--strand-specific"
                    },
                    "doc": "The RNA-Seq protocol used to generate the reads is strand specific,\ni.e., all (upstream) reads are derived from the forward strand. This\noption is equivalent to --forward-prob=1.0. With this option set, if\nRSEM runs the Bowtie/Bowtie 2 aligner, the '--norc' Bowtie/Bowtie 2\noption will be used, which disables alignment to the reverse strand\nof transcripts. (Default: off)\n",
                    "id": "#rsem-calculate-expression.cwl/strand-specific"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--tag"
                    },
                    "doc": "<string>\nThe name of the optional field used in the SAM input for identifying\na read with too many valid alignments. The field should have the\nformat <tagName>:i:<value>, where a <value> bigger than 0 indicates\na read with too many alignments. (Default: \"\")\n",
                    "id": "#rsem-calculate-expression.cwl/tag"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--temporary-folder"
                    },
                    "doc": "<string>\nSet where to put the temporary files generated by RSEM. If the\nfolder specified does not exist, RSEM will try to create it.\n(Default: sample_name.temp)\n",
                    "id": "#rsem-calculate-expression.cwl/temporary-folder"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--time"
                    },
                    "doc": "Output time consumed by each step of RSEM to 'sample_name.time'.\n(Default: off)\n",
                    "id": "#rsem-calculate-expression.cwl/time"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "position": 2,
                        "itemSeparator": ","
                    },
                    "doc": "Comma-separated list of files containing single-end reads or\nupstream reads for paired-end data. By default, these files are\nassumed to be in FASTQ format. If the --no-qualities option is\nspecified, then FASTA format is expected.\n",
                    "id": "#rsem-calculate-expression.cwl/upstream_read_files"
                }
            ],
            "outputs": [
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.sample_name + \".alleles.results\")"
                    },
                    "doc": "Only generated when the RSEM references are built with\nallele-specific transcripts.\nThis file contains allele level expression estimates for\nallele-specific expression calculation. The first line contains\ncolumn names separated by the tab character. The format of each line\nin the rest of this file is:\nallele_id transcript_id gene_id length effective_length\nexpected_count TPM FPKM AlleleIsoPct AlleleGenePct\n[posterior_mean_count posterior_standard_deviation_of_count pme_TPM\npme_FPKM AlleleIsoPct_from_pme_TPM AlleleGenePct_from_pme_TPM\nTPM_ci_lower_bound TPM_ci_upper_bound FPKM_ci_lower_bound\nFPKM_ci_upper_bound]\nFields are separated by the tab character. Fields within \"[]\" are\noptional. They will not be presented if neither '--calc-pme' nor\n'--calc-ci' is set.\n'allele_id' is the allele-specific name of this allele-specific\ntranscript.\n'AlleleIsoPct' stands for allele-specific percentage on isoform\nlevel. It is the percentage of this allele-specific transcript's\nabundance over its parent transcript's abundance. If its parent\ntranscript has only one allele variant form, this field will be set\nto 100.\n'AlleleGenePct' stands for allele-specific percentage on gene level.\nIt is the percentage of this allele-specific transcript's abundance\nover its parent gene's abundance.\n'AlleleIsoPct_from_pme_TPM' and 'AlleleGenePct_from_pme_TPM' have\nsimilar meanings. They are calculated based on posterior mean\nestimates.\nPlease note that if this file is present, the fields 'length' and\n'effective_length' in 'sample_name.isoforms.results' should be\ninterpreted similarly as the corresponding definitions in\n'sample_name.genes.results'.\n",
                    "id": "#rsem-calculate-expression.cwl/alleles"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.sample_name + \".genes.results\")"
                    },
                    "doc": "File containing gene level expression estimates. The first line\ncontains column names separated by the tab character. The format of\neach line in the rest of this file is:\ngene_id transcript_id(s) length effective_length expected_count TPM\nFPKM [posterior_mean_count posterior_standard_deviation_of_count\npme_TPM pme_FPKM TPM_ci_lower_bound TPM_ci_upper_bound\nFPKM_ci_lower_bound FPKM_ci_upper_bound]\nFields are separated by the tab character. Fields within \"[]\" are\noptional. They will not be presented if neither '--calc-pme' nor\n'--calc-ci' is set.\n'transcript_id(s)' is a comma-separated list of transcript_ids\nbelonging to this gene. If no gene information is provided,\n'gene_id' and 'transcript_id(s)' are identical (the\n'transcript_id').\nA gene's 'length' and 'effective_length' are defined as the weighted\naverage of its transcripts' lengths and effective lengths (weighted\nby 'IsoPct'). A gene's abundance estimates are just the sum of its\ntranscripts' abundance estimates.\n",
                    "id": "#rsem-calculate-expression.cwl/genes"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.sample_name + \".isoforms.results\")"
                    },
                    "doc": "File containing isoform level expression estimates. The first line\ncontains column names separated by the tab character. The format of\neach line in the rest of this file is:\ntranscript_id gene_id length effective_length expected_count TPM\nFPKM IsoPct [posterior_mean_count\nposterior_standard_deviation_of_count pme_TPM pme_FPKM\nIsoPct_from_pme_TPM TPM_ci_lower_bound TPM_ci_upper_bound\nFPKM_ci_lower_bound FPKM_ci_upper_bound]\nFields are separated by the tab character. Fields within \"[]\" are\noptional. They will not be presented if neither '--calc-pme' nor\n'--calc-ci' is set.\n'transcript_id' is the transcript name of this transcript. 'gene_id'\nis the gene name of the gene which this transcript belongs to\n(denote this gene as its parent gene). If no gene information is\nprovided, 'gene_id' and 'transcript_id' are the same.\n'length' is this transcript's sequence length (poly(A) tail is not\ncounted). 'effective_length' counts only the positions that can\ngenerate a valid fragment. If no poly(A) tail is added,\n'effective_length' is equal to transcript length - mean fragment\nlength + 1. If one transcript's effective length is less than 1,\nthis transcript's both effective length and abundance estimates are\nset to 0.\n'expected_count' is the sum of the posterior probability of each\nread comes from this transcript over all reads. Because 1) each read\naligning to this transcript has a probability of being generated\nfrom background noise; 2) RSEM may filter some alignable low quality\nreads, the sum of expected counts for all transcript are generally\nless than the total number of reads aligned.\n'TPM' stands for Transcripts Per Million. It is a relative measure\nof transcript abundance. The sum of all transcripts' TPM is 1\nmillion. 'FPKM' stands for Fragments Per Kilobase of transcript per\nMillion mapped reads. It is another relative measure of transcript\nabundance. If we define l_bar be the mean transcript length in a\nsample, which can be calculated as\nl_bar = \\sum_i TPM_i / 10^6 * effective_length_i (i goes through\nevery transcript),\nthe following equation is hold:\nFPKM_i = 10^3 / l_bar * TPM_i.\nWe can see that the sum of FPKM is not a constant across samples.\n'IsoPct' stands for isoform percentage. It is the percentage of this\ntranscript's abandunce over its parent gene's abandunce. If its\nparent gene has only one isoform or the gene information is not\nprovided, this field will be set to 100.\n'posterior_mean_count', 'pme_TPM', 'pme_FPKM' are posterior mean\nestimates calculated by RSEM's Gibbs sampler.\n'posterior_standard_deviation_of_count' is the posterior standard\ndeviation of counts. 'IsoPct_from_pme_TPM' is the isoform percentage\ncalculated from 'pme_TPM' values.\n'TPM_ci_lower_bound', 'TPM_ci_upper_bound', 'FPKM_ci_lower_bound'\nand 'FPKM_ci_upper_bound' are lower(l) and upper(u) bounds of 95%\ncredibility intervals for TPM and FPKM values. The bounds are\ninclusive (i.e. [l, u]).\n",
                    "id": "#rsem-calculate-expression.cwl/isoforms"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.sample_name + \".stat/*\")"
                    },
                    "doc": "This is a folder instead of a file. All model related statistics are\nstored in this folder. Use 'rsem-plot-model' can generate plots\nusing this folder.\n'sample_name.stat/sample_name.cnt' contains alignment statistics.\nThe format and meanings of each field are described in\n'cnt_file_description.txt' under RSEM directory.\n'sample_name.stat/sample_name.model' stores RNA-Seq model parameters\nlearned from the data. The format and meanings of each filed of this\nfile are described in 'model_file_description.txt' under RSEM\ndirectory.\n",
                    "id": "#rsem-calculate-expression.cwl/rsem_stat"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.sample_name + \".time\")"
                    },
                    "doc": "Only generated when --time is specified.\nIt contains time (in seconds) consumed by aligning reads, estimating\nexpression levels and calculating credibility intervals.\n",
                    "id": "#rsem-calculate-expression.cwl/rsem_time"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.sample_name + \".transcript.bam\")"
                    },
                    "doc": "BAM-formatted file of read\nalignments in transcript coordinates. The MAPQ field of each\nalignment is set to min(100, floor(-10 * log10(1.0 - w) + 0.5)),\nwhere w is the posterior probability of that alignment being the\ntrue mapping of a read. In addition, RSEM pads a new tag ZW:f:value,\nwhere value is a single precision floating number representing the\nposterior probability. Because this file contains all alignment\nlines produced by bowtie or user-specified aligners, it can also be\nused as a replacement of the aligner generated BAM/SAM file. For\npaired-end reads, if one mate has alignments but the other does not,\nthis file marks the alignable mate as \"unmappable\" (flag bit 0x4)\nand appends an optional field \"Z0:A:!\".\n",
                    "id": "#rsem-calculate-expression.cwl/transcript_bam"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.sample_name + \".transcript.sorted.bam\")"
                    },
                    "id": "#rsem-calculate-expression.cwl/transcript_sorted_bam"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.sample_name + \".transcript.sorted.bam.bai\")"
                    },
                    "id": "#rsem-calculate-expression.cwl/transcript_sorted_bam_bai"
                }
            ],
            "baseCommand": [
                "rsem-calculate-expression"
            ],
            "id": "#rsem-calculate-expression.cwl"
        },
        {
            "class": "Workflow",
            "doc": "Split reads in a BAM file by strands and index forward and reverse output BAM files",
            "requirements": [
                {
                    "class": "ScatterFeatureRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "doc": "JSON filter file for forward strand used in bamtools (see bamtools-filter command)",
                    "type": "File",
                    "id": "#split-bams-by-strand-and-index.cwl/bamtools_forward_filter_file"
                },
                {
                    "doc": "JSON filter file for reverse strand used in bamtools (see bamtools-filter command)",
                    "type": "File",
                    "id": "#split-bams-by-strand-and-index.cwl/bamtools_reverse_filter_file"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "id": "#split-bams-by-strand-and-index.cwl/input_bam_files"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "string"
                    },
                    "id": "#split-bams-by-strand-and-index.cwl/input_basenames"
                }
            ],
            "steps": [
                {
                    "run": "#samtools-index.cwl",
                    "in": [
                        {
                            "source": "#split-bams-by-strand-and-index.cwl/split-bam-minus/output_file",
                            "id": "#split-bams-by-strand-and-index.cwl/index_minus_bam/input_file"
                        }
                    ],
                    "scatter": "#split-bams-by-strand-and-index.cwl/index_minus_bam/input_file",
                    "out": [
                        "#split-bams-by-strand-and-index.cwl/index_minus_bam/indexed_file"
                    ],
                    "id": "#split-bams-by-strand-and-index.cwl/index_minus_bam"
                },
                {
                    "run": "#samtools-index.cwl",
                    "scatter": "#split-bams-by-strand-and-index.cwl/index_plus_bam/input_file",
                    "in": [
                        {
                            "source": "#split-bams-by-strand-and-index.cwl/split-bam-plus/output_file",
                            "id": "#split-bams-by-strand-and-index.cwl/index_plus_bam/input_file"
                        }
                    ],
                    "out": [
                        "#split-bams-by-strand-and-index.cwl/index_plus_bam/indexed_file"
                    ],
                    "id": "#split-bams-by-strand-and-index.cwl/index_plus_bam"
                },
                {
                    "run": "#bamtools-filter.cwl",
                    "scatterMethod": "dotproduct",
                    "scatter": [
                        "#split-bams-by-strand-and-index.cwl/split-bam-minus/in",
                        "#split-bams-by-strand-and-index.cwl/split-bam-minus/out"
                    ],
                    "in": [
                        {
                            "source": "#split-bams-by-strand-and-index.cwl/input_bam_files",
                            "valueFrom": "${return [self]}",
                            "id": "#split-bams-by-strand-and-index.cwl/split-bam-minus/in"
                        },
                        {
                            "source": "#split-bams-by-strand-and-index.cwl/input_basenames",
                            "valueFrom": "$(self + \".Aligned.minus.bam\")",
                            "id": "#split-bams-by-strand-and-index.cwl/split-bam-minus/out"
                        },
                        {
                            "source": "#split-bams-by-strand-and-index.cwl/bamtools_reverse_filter_file",
                            "id": "#split-bams-by-strand-and-index.cwl/split-bam-minus/script"
                        }
                    ],
                    "out": [
                        "#split-bams-by-strand-and-index.cwl/split-bam-minus/output_file"
                    ],
                    "id": "#split-bams-by-strand-and-index.cwl/split-bam-minus"
                },
                {
                    "run": "#bamtools-filter.cwl",
                    "scatterMethod": "dotproduct",
                    "scatter": [
                        "#split-bams-by-strand-and-index.cwl/split-bam-plus/in",
                        "#split-bams-by-strand-and-index.cwl/split-bam-plus/out"
                    ],
                    "in": [
                        {
                            "source": "#split-bams-by-strand-and-index.cwl/input_bam_files",
                            "valueFrom": "${return [self]}",
                            "id": "#split-bams-by-strand-and-index.cwl/split-bam-plus/in"
                        },
                        {
                            "source": "#split-bams-by-strand-and-index.cwl/input_basenames",
                            "valueFrom": "$(self + \".Aligned.plus.bam\")",
                            "id": "#split-bams-by-strand-and-index.cwl/split-bam-plus/out"
                        },
                        {
                            "source": "#split-bams-by-strand-and-index.cwl/bamtools_forward_filter_file",
                            "id": "#split-bams-by-strand-and-index.cwl/split-bam-plus/script"
                        }
                    ],
                    "out": [
                        "#split-bams-by-strand-and-index.cwl/split-bam-plus/output_file"
                    ],
                    "id": "#split-bams-by-strand-and-index.cwl/split-bam-plus"
                }
            ],
            "outputs": [
                {
                    "doc": "BAM files containing only reads in the reverse (minus) strand.",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#split-bams-by-strand-and-index.cwl/index_minus_bam/indexed_file",
                    "id": "#split-bams-by-strand-and-index.cwl/bam_minus_files"
                },
                {
                    "doc": "BAM files containing only reads in the forward (plus) strand.",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#split-bams-by-strand-and-index.cwl/index_plus_bam/indexed_file",
                    "id": "#split-bams-by-strand-and-index.cwl/bam_plus_files"
                }
            ],
            "id": "#split-bams-by-strand-and-index.cwl"
        },
        {
            "class": "CommandLineTool",
            "doc": "featureCounts is a highly efficient general-purpose read summarization program that counts mapped reads for genomic features such as genes, exons, promoter, gene bodies, genomic bins and chromosomal locations. It can be used to count both RNA-seq and genomic DNA-seq reads.",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "reddylab/subread:1.4.6-p4",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-A"
                    },
                    "doc": "<input>\tSpecify the name of a file including aliases of chromosome\nnames. The file should be a comma delimited text file that\nincludes two columns. The first column gives the chromosome\nnames used in the annotation and the second column gives the\nchromosome names used by reads. This file should not contain\nheader lines. Names included in this file are case sensitive.\n",
                    "id": "#subread-featurecounts.cwl/A"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-B"
                    },
                    "doc": "       \tIf specified, only fragments that have both ends \nsuccessfully aligned will be considered for summarization.\nThis option is only applicable for paired-end reads.\n",
                    "id": "#subread-featurecounts.cwl/B"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-C"
                    },
                    "doc": "       \tIf specified, the chimeric fragments (those fragments that \nhave their two ends aligned to different chromosomes) will\nNOT be included for summarization. This option is only\napplicable for paired-end read data.\n",
                    "id": "#subread-featurecounts.cwl/C"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-D"
                    },
                    "doc": "<int>  \tMaximum fragment/template length, 600 by default.\n",
                    "id": "#subread-featurecounts.cwl/D"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-F"
                    },
                    "doc": "<input>\tSpecify the format of the annotation file. Acceptable formats\ninclude `GTF' and `SAF'. `GTF' by default. Please refer to the\nusers guide for SAF annotation format.\n",
                    "id": "#subread-featurecounts.cwl/F"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-M"
                    },
                    "doc": "       \tIf specified, multi-mapping reads/fragments will be counted (ie.\na multi-mapping read will be counted up to N times if it has N\nreported mapping locations). The program uses the `NH' tag to\nfind multi-mapping reads.\n",
                    "id": "#subread-featurecounts.cwl/M"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-O"
                    },
                    "doc": "       \tIf specified, reads (or fragments if -p is specified) will\nbe allowed to be assigned to more than one matched meta-\nfeature (or feature if -f is specified).\n",
                    "id": "#subread-featurecounts.cwl/O"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-P"
                    },
                    "doc": "       \tIf specified, paired-end distance will be checked when assigning\nfragments to meta-features or features. This option is only\napplicable when -p is specified. The distance thresholds should\nbe specified using -d and -D options.\n",
                    "id": "#subread-featurecounts.cwl/P"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-Q"
                    },
                    "doc": "<int>  \tThe minimum mapping quality score a read must satisfy in order\nto be counted. For paired-end reads, at least one end should\nsatisfy this criteria. 0 by default.\n",
                    "id": "#subread-featurecounts.cwl/Q"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-T"
                    },
                    "doc": "<int>  \tNumber of the threads. 1 by default.\n",
                    "id": "#subread-featurecounts.cwl/T"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 2,
                        "prefix": "-a"
                    },
                    "doc": "-a <input>\nGive the name of the annotation file. The program assumes\nthat the provided annotation file is in GTF format. Use -F\noption to specify other annotation formats.\n",
                    "id": "#subread-featurecounts.cwl/annotation_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--countSplitAlignmentsOnly"
                    },
                    "doc": "\tIf specified, only split alignments (CIGAR\nstrings containing letter `N') will be counted. All the other\nalignments will be ignored. An example of split alignments is\nthe exon-spanning reads in RNA-seq data.\nOptional paired-end parameters:\n",
                    "id": "#subread-featurecounts.cwl/countSplitAlignmentsOnly"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-d"
                    },
                    "doc": "<int>  \tMinimum fragment/template length, 50 by default.\n",
                    "id": "#subread-featurecounts.cwl/d"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--donotsort"
                    },
                    "doc": "  If specified, paired end reads will not be reordered even if\nreads from the same pair were found not to be next to each other\nin the input.\n",
                    "id": "#subread-featurecounts.cwl/donotsort"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-f"
                    },
                    "doc": "       \tIf specified, read summarization will be performed at the \nfeature level (eg. exon level). Otherwise, it is performed at\nmeta-feature level (eg. gene level).\n",
                    "id": "#subread-featurecounts.cwl/f"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--fraction"
                    },
                    "doc": "If specified, a fractional count 1/n will be generated for each\nmulti-mapping read, where n is the number of alignments (indica-\nted by 'NH' tag) reported for the read. This option must be used\ntogether with the '-M' option.\n",
                    "id": "#subread-featurecounts.cwl/fraction"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-g"
                    },
                    "doc": "<input>\tSpecify the attribute type used to group features (eg. exons)\ninto meta-features (eg. genes), when GTF annotation is provided.\n`gene_id' by default. This attribute type is usually the gene\nidentifier. This argument is useful for the meta-feature level\nsummarization.\n",
                    "id": "#subread-featurecounts.cwl/g"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--ignoreDup"
                    },
                    "doc": "\tIf specified, reads that were marked as\nduplicates will be ignored. Bit Ox400 in FLAG field of SAM/BAM\nfile is used for identifying duplicate reads. In paired end\ndata, the entire read pair will be ignored if at least one end\nis found to be a duplicate read.\n",
                    "id": "#subread-featurecounts.cwl/ignoreDup"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "inputBinding": {
                        "position": 4
                    },
                    "doc": "<input_files>\nGive the names of input read files that include the read\nmapping results. Format of input files is automatically\ndetermined (SAM or BAM). Paired-end reads will be\nautomatically re-ordered if it is found that reads from the\nsame pair are not adjacent to each other. Multiple files can\nbe provided at the same time.\n",
                    "id": "#subread-featurecounts.cwl/input_files"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--minReadOverlap"
                    },
                    "doc": "<int>      Specify the minimum number of overlapped bases\nrequired to assign a read to a feature. 1 by default. Negative\nvalues are permitted, indicating a gap being allowed between a\nread and a feature.\n",
                    "id": "#subread-featurecounts.cwl/minReadOverlap"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 3,
                        "prefix": "-o"
                    },
                    "doc": "-o <input>\nGive the name of the output file. The output file contains\nthe number of reads assigned to each meta-feature (or each\nfeature if -f is specified). A meta-feature is the aggregation\nof features, grouped by using gene identifiers. Please refer\nto the users guide for more details.\n",
                    "id": "#subread-featurecounts.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-p"
                    },
                    "doc": "       \tIf specified, fragments (or templates) will be counted instead\nof reads. This option is only applicable for paired-end reads.\nThe two reads from the same fragment must be adjacent to each\nother in the provided SAM/BAM file.\n",
                    "id": "#subread-featurecounts.cwl/p"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--primary"
                    },
                    "doc": "\tIf specified, only primary alignments will be counted. Primary\nand secondary alignments are identified using bit 0x100 in the\nFlag field of SAM/BAM files. All primary alignments in a dataset\nwill be counted no matter they are from multi-mapping reads or\nnot ('-M' is ignored).\n",
                    "id": "#subread-featurecounts.cwl/primary"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--read2pos"
                    },
                    "doc": "<5:3>            The read is reduced to its 5' most base or 3'\nmost base. Read summarization is then performed based on the\nsingle base which the read is reduced to.\n",
                    "id": "#subread-featurecounts.cwl/read2pos"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--readExtension3"
                    },
                    "doc": "<int>      Reads are extended upstream by <int> bases from\ntheir 3' end.\n",
                    "id": "#subread-featurecounts.cwl/readExtension3"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--readExtension5"
                    },
                    "doc": "<int>      Reads are extended upstream by <int> bases from\ntheir 5' end.\n",
                    "id": "#subread-featurecounts.cwl/readExtension5"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-s"
                    },
                    "doc": "<int>  \tIndicate if strand-specific read counting should be performed.\nIt has three possible values:  0 (unstranded), 1 (stranded) and\n2 (reversely stranded). 0 by default.\n",
                    "id": "#subread-featurecounts.cwl/s"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-t"
                    },
                    "doc": "<input>\tSpecify the feature type. Only rows which have the matched\nmatched feature type in the provided GTF annotation file\nwill be included for read counting. `exon' by default.\n",
                    "id": "#subread-featurecounts.cwl/t"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-v"
                    },
                    "doc": "       \tOutput version of the program.\n",
                    "id": "#subread-featurecounts.cwl/v"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_filename)"
                    },
                    "id": "#subread-featurecounts.cwl/output_files"
                }
            ],
            "baseCommand": [
                "featureCounts"
            ],
            "id": "#subread-featurecounts.cwl"
        },
        {
            "class": "ExpressionTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "string",
                    "id": "#basename.cwl/file_path"
                },
                {
                    "type": "int",
                    "default": 1,
                    "doc": "Number of file extensions to remove",
                    "id": "#basename.cwl/num_ext"
                },
                {
                    "type": "string",
                    "default": ".",
                    "doc": "File extension delimiter",
                    "id": "#basename.cwl/sep"
                }
            ],
            "expression": "${\n  var out=inputs.file_path.replace(/^.*[\\\\\\/]/, '');\n  for (var e=0;e<inputs.num_ext; e++){\n    out=out.split(RegExp(inputs.sep))[0];\n  }\n  return {\"basename\": out};\n}\n",
            "outputs": [
                {
                    "type": "string",
                    "id": "#basename.cwl/basename"
                }
            ],
            "id": "#basename.cwl"
        }
    ],
    "cwlVersion": "v1.0"
}
